@@ -12,14 +12,18 @@ from typing import Dict, List, Optional
 from toml import load as toml_load
 
 from yd_commands.args import ARGS_PARSER
-from yd_commands.config import ConfigWorkRequirement
-from yd_commands.config_keys import *
+from yd_commands.config_types import ConfigWorkRequirement
 from yd_commands.printing import print_json, print_log
+from yd_commands.property_names import *
+from yd_commands.settings import (
+    BOOL_TYPE_TAG,
+    CSV_VAR_CLOSING_DELIMITER,
+    CSV_VAR_OPENING_DELIMITER,
+    NUMBER_TYPE_TAG,
+)
 from yd_commands.variables import (
-    BOOL_SUB,
-    NUMBER_SUB,
     load_jsonnet_file_with_variable_substitutions,
-    process_variable_substitutions,
+    process_variable_substitutions_insitu,
 )
 
 
@@ -216,7 +220,7 @@ def perform_csv_task_expansion(wr_data: Dict, csv_files: List[str]) -> Dict:
         exit(0)
 
     # Process remaining substitutions
-    process_variable_substitutions(wr_data)
+    process_variable_substitutions_insitu(wr_data)
     return wr_data
 
 
@@ -238,9 +242,11 @@ def make_string_substitutions(input: str, var_name: str, value: str) -> str:
     """
     Helper function to make string substitutions for CSV variables only.
     """
-    input = input.replace(f"{{{{{var_name}}}}}", value)
+    input = input.replace(
+        f"{CSV_VAR_OPENING_DELIMITER}{var_name}{CSV_VAR_CLOSING_DELIMITER}", value
+    )
 
-    num_sub_str = f"{{{{{NUMBER_SUB}{var_name}}}}}"
+    num_sub_str = f"{CSV_VAR_OPENING_DELIMITER}{NUMBER_TYPE_TAG}{var_name}{CSV_VAR_CLOSING_DELIMITER}"
     if num_sub_str in input:
         try:
             float(value)
@@ -248,7 +254,7 @@ def make_string_substitutions(input: str, var_name: str, value: str) -> str:
             raise Exception(f"Invalid number substitution in CSV: '{value}'")
         input = input.replace(f"'{num_sub_str}'", value)
 
-    bool_sub_str = f"{{{{{BOOL_SUB}{var_name}}}}}"
+    bool_sub_str = f"{CSV_VAR_OPENING_DELIMITER}{BOOL_TYPE_TAG}{var_name}{CSV_VAR_CLOSING_DELIMITER}"
     if bool_sub_str in input:
         if value.lower() == "true":
             value = "True"
@@ -314,13 +320,20 @@ def substitions_present(var_names: List[str], task_prototype: str) -> bool:
     Check if there are any CSV substitutions present in the Task prototype.
     """
     return (
-        any(f"{{{{{var_name}}}}}" in task_prototype for var_name in var_names)
-        or any(
-            f"{{{{{NUMBER_SUB}{var_name}}}}}" in task_prototype
+        any(
+            f"{CSV_VAR_OPENING_DELIMITER}{var_name}{CSV_VAR_CLOSING_DELIMITER}"
+            in task_prototype
             for var_name in var_names
         )
         or any(
-            f"{{{{{BOOL_SUB}{var_name}}}}}" in task_prototype for var_name in var_names
+            f"{CSV_VAR_OPENING_DELIMITER}{NUMBER_TYPE_TAG}{var_name}{CSV_VAR_CLOSING_DELIMITER}"
+            in task_prototype
+            for var_name in var_names
+        )
+        or any(
+            f"{CSV_VAR_OPENING_DELIMITER}{BOOL_TYPE_TAG}{var_name}{CSV_VAR_CLOSING_DELIMITER}"
+            in task_prototype
+            for var_name in var_names
         )
     )
 
@@ -335,6 +348,7 @@ def csv_expand_toml_tasks(config_wr: ConfigWorkRequirement, csv_file: str) -> Di
     csv_data = CSV_DATA_CACHE.get_csv_task_data(csv_file.split(":")[0])
     # Populate properties that can be set at Task level only
     for config_value, config_name in [
+        (config_wr.always_upload, ALWAYS_UPLOAD),
         (config_wr.args, ARGS),
         (config_wr.capture_taskoutput, CAPTURE_TASKOUTPUT),
         (config_wr.docker_env, DOCKER_ENV),
@@ -343,10 +357,11 @@ def csv_expand_toml_tasks(config_wr: ConfigWorkRequirement, csv_file: str) -> Di
         (config_wr.env, ENV),
         (config_wr.executable, EXECUTABLE),
         (config_wr.flatten_input_paths, FLATTEN_PATHS),
-        (config_wr.input_files, INPUT_FILES),
-        (config_wr.optional_inputs, INPUTS_OPTIONAL),
-        (config_wr.output_files, OUTPUT_FILES),
-        (config_wr.output_files_required, OUTPUT_FILES_REQUIRED),
+        (config_wr.inputs_optional, INPUTS_OPTIONAL),
+        (config_wr.inputs_required, INPUTS_REQUIRED),
+        (config_wr.outputs_optional, OUTPUTS_OPTIONAL),
+        (config_wr.outputs_other, OUTPUTS_OTHER),
+        (config_wr.outputs_required, OUTPUTS_REQUIRED),
         (config_wr.task_data, TASK_DATA),
         (config_wr.task_data_file, TASK_DATA_FILE),
         (config_wr.task_group_name, TASK_GROUP_NAME),  # Note: oddity
@@ -355,6 +370,7 @@ def csv_expand_toml_tasks(config_wr: ConfigWorkRequirement, csv_file: str) -> Di
         (config_wr.upload_files, UPLOAD_FILES),
         (config_wr.verify_at_start, VERIFY_AT_START),
         (config_wr.verify_wait, VERIFY_WAIT),
+        # Note: not TASK_COUNT; count determined by CSV data
     ]:
         if config_value is not None and substitions_present(
             csv_data.var_names, str(config_value)
